/**************************************************************************//**
 *
 * Copyright 1998-2012 NetBurner, Inc.  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or it's derivatives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non-NetBurner Hardware.
 *   Please contact sales@Netburner.com for more information.
 *
 *   NetBurner makes no representation or warranties
 *   with respect to the performance of this computer program, and
 *   specifically disclaims any responsibility for any damages,
 *   special or consequential, connected with the use of this program.
 *
 *---------------------------------------------------------------------
 * NetBurner, Inc.
 * 5405 Morehouse Drive
 * San Diego, California 92121
 *
 * information available at:  http://www.netburner.com
 * E-Mail info@netburner.com
 *
 * Support is available: E-Mail support@netburner.com
 *
 *****************************************************************************/

#ifndef  _DMA_SPI_H_INC
#define  _DMA_SPI_H_INC
#include <basictypes.h>


#ifndef __cplusplus
#error DSPI driver is a C++ only library
#endif

/*
   DSPI state
*/   
#define DSPI_OK      ( 0 )
#define DSPI_BUSY    ( 1 )


// MCR Register masks
#define MCR_MASTER_INIT     0x80000C01
#define MCR_HALT_BIT        0x00000001

// CTAR Register masks
#define CTAR_CLOCK_POLARITY 0x04000000
#define CTAR_CLOCK_PHASE    0x02000000
#define CTAR_FRAME_16BIT    0x78000000
#define CTAR_FRAME_4BIT     0x18000000

// SR Register masks
#define SR_EOQF_MASK        0x10000000
#define SR_CLR_FLAGS        0x9A0A0000 

// RSER (Interrupt/DMA enable) Register masks
#define RSER_EOQF_IRQ_ONLY  ~0x8B0B0000

// PUSHR (command) Register masks
#define PUSHR_CONT_BIT      0x80000000
#define PUSHR_EOQ_BIT       0x08000000


#define DEFAULT_DSPI_MODULE 1
#define DSPI_MODULE_COUNT   4


/*
 *****************************************************************************-
 *
 *   dspiDriverStruct 
 *
 *   This struct contains the major variables/configurations used for a DSPI transfer
 *    
 *   volatile BYTE* pDSPIRxbuf/pDSPITxbuf -
 *   These pointers are used to track the locations in memory where data will be 
 *   read or written to the peripheral    
 *    
 *   BYTE BitsPerQueue - This is the number if bits per transfer, (value = 8 - 32) 
 *    
 *   DWORD DSPI_SizeLeft - This is the number if bytes left in the transfer
 *
 *   WORD Command_Mask - This is a partial configuration for the queue's command reg
 *
 *   OS_SEM* DSPI_Sem - This is a pointer to an external semaphore provided by DSPIStart() 
 *
 *   BYTE DSPI_INT_STATUS - Status of the spi device
 *
 *   DWORD WordsToWrite - Number of words to write to the command/TX Queues and Read from RX
 *
 *   DWORD LastWordsToWrite - the amount from the previous ISR
 *
 *   BOOL DSPIfinished - Flag indicating the requested spi transmissions are complete
 *         
 *****************************************************************************-
 */
typedef struct 
{  
    volatile BYTE* pDSPIRxbuf;               
    volatile BYTE* pDSPITxbuf;
    BYTE BitsPerQueue;
    DWORD DSPI_SizeLeft;
    WORD Command_Mask;
    OS_SEM* DSPI_Sem;
    volatile BYTE DSPI_INT_STATUS;
    DWORD WordsToWrite;
    DWORD LastWordsToWrite;
    volatile BOOL DSPIfinished;
} dspiDriverStruct;

/*
 *****************************************************************************-
 *
 *  Initialize DMA Serial Peripheral Interface (DSPI)
 *
 *      Parameters:
 *          baudRateInBps          - Maximum speed requested. (1) (2)
 *          transferSizeInBits     - 8 thru 32 valid (3)
 *          SPIModule              - the dspi module to target on the processor
 *          peripheralChipSelects  - DSPI_CS[7:0] (4)
 *          chipSelectPolarity     - CS[7:0] 0 inactive logic level low, 1 high
 *          clockPolarity          - 0 inactive logic level low, 1 high
 *          clockPhase             - 0 (5) or 1 (6)
 *          doutHiz                - Data output high impedance between transfers
 *          csToClockDelay         - Delay form chip select to valid clock (7)
 *          delayAfterTransfer     - Delay between data transfers (8)
 *
 *      Return:
 *          Current DSPI state     - DSPI_OK success or DSPI_BUSY busy
 *
 *      Notes:
 *          (1) Maximum BAUD rate is CPU_CLOCK / 3
 *          (2) Will select highest BAUD rate available that does not exceed.
 *          (3) Transfers > 8 must be word aligned
 *          (4) Select based on chipSelectPolarity  
 *          (5) 0 data captured leading edge of DSPI_CLK, changed following edge.
 *          (6) 1 data changed leading edge of DSPI_CLK, captured following edge.
 *          (7) 0 default is as close to 1/2 DSPI_CLK without going under, 
 *              keeping with the interface to QSPI
 *          (8) 0 default is 17/(system clock / 2), in keepin with interface to QSPI
 *
 *****************************************************************************-
 */
 
BYTE DSPIInit( BYTE SPIModule = DEFAULT_DSPI_MODULE, DWORD baudRateInBps = 2000000, 
        BYTE transferSizeInBits = 8, BYTE peripheralChipSelects = 0x0F, 
        BYTE chipSelectPolarity = 0x0F, BYTE clockPolarity = 0, 
        BYTE clockPhase = 1, BOOL doutHiz = TRUE, 
        BYTE csToClockDelay = 0, BYTE delayAfterTransfer = 0 );  

/*
 *****************************************************************************-
 *
 *  Start DSPI Data Transfer
 *
 *      Parameters:
 *          transmitBufferPtr   - Buffer with data to send, NULL for receiving
 *          receiveBufferPtr    - Buffer to receive data, NULL for sending
 *          byteCount           - Count of bytes to send or receive
 *          SPIModule           - Selects the dspi module to use
 *          finishedSem         - Optional semaphore to set when completed
 *
 *      Return:
 *          Current DSPI state  - DSPI_OK success or DSPI_BUSY busy
 *
 *      Notes:
 *          None
 *
 *****************************************************************************-
 */
 
BYTE DSPIStart( BYTE SPIModule, PBYTE transmitBufferPtr, volatile BYTE* receiveBufferPtr, 
      DWORD byteCount, OS_SEM* finishedSem = NULL );

/*
 *****************************************************************************-
 *
 * Check current DSPI Data Transfer
 *
 *    Parameters:
 *       None
 *
 *    Return:
 *       Progress of transfer    - TRUE is finished, FALSE is active
 *
 *    Notes:
 *       None
 *
 *****************************************************************************-
 */
 
BOOL DSPIdone( BYTE SPIModule = DEFAULT_DSPI_MODULE );

// QSPI to DSPITranslation macros
//  Translates QSPI calls to DSPI calls

inline BYTE QSPIInit( DWORD baudRateInBps = 2000000, BYTE transferSizeInBits = 8, 
      BYTE peripheralChipSelects = 0x0F, BYTE chipSelectPolarity = 1, 
      BYTE clockPolarity = 0, BYTE clockPhase = 1, BOOL doutHiz = TRUE, 
      BYTE csToClockDelay = 0, BYTE delayAfterTransfer = 0 )
{
    return DSPIInit( DEFAULT_DSPI_MODULE, baudRateInBps, transferSizeInBits, 
                peripheralChipSelects, chipSelectPolarity, clockPolarity, clockPhase, 
                doutHiz, csToClockDelay, delayAfterTransfer );
}

inline BYTE QSPIStart( PBYTE transmitBufferPtr, volatile BYTE* receiveBufferPtr, 
      DWORD byteCount, OS_SEM* finishedSem = NULL )
{
    return DSPIStart( DEFAULT_DSPI_MODULE, transmitBufferPtr, receiveBufferPtr, 
                byteCount, finishedSem );
}

inline BOOL QSPIdone( )
{
    return DSPIdone( );
}

#endif   /* ----- #ifndef _DMA_SPI_H_INC  ----- */
